// src/shared.ts
var chunked = (array, step) => {
  const chunks = [];
  for (let index = 0; index < array.length; index += step) {
    chunks.push(array.slice(index, index + step));
  }
  return chunks;
};
var randomInt = (a = 1, b = 0) => {
  const lower = Math.ceil(Math.min(a, b));
  const upper = Math.floor(Math.max(a, b));
  return Math.floor(lower + Math.random() * (upper - lower + 1));
};

// src/discord/helper.ts
var youtubeChannelLinkFromChannelId = (channelId) => `https://youtube.com/channel/${channelId}`;
var youtubeVideoLinkFromVideoId = (videoId) => `https://www.youtube.com/watch?v=${videoId}`;
var timestampMap = {
  shortTime: "t",
  longTime: "T",
  shortDate: "d",
  longDate: "D",
  shortDateTime: "f",
  longDateTime: "F",
  relativeTime: "R"
};
var discordUnixTimestamp = (n, style) => `<t:${n / 1e3}:${timestampMap[style ?? "shortDateTime"]}>`;
var mentionMap = {
  channel: "#",
  role: "@&",
  user: "@"
};
var discordMention = (id, type) => `<${mentionMap[type]}${id}>`;

// src/discord/embed-builder/transformer.ts
var defaultTimestamp = (n) => {
  const absoluteTimestamp = discordUnixTimestamp(n, "shortDateTime");
  const relativeTimestamp = discordUnixTimestamp(n, "relativeTime");
  return `**${absoluteTimestamp} | ${relativeTimestamp}**`;
};
var rangeTimestamp = (from, to) => {
  const fromAbsoluteTimestamp = discordUnixTimestamp(from, "shortDateTime");
  const toAbsoluteTimestamp = discordUnixTimestamp(to, "shortDateTime");
  return `**${fromAbsoluteTimestamp} - ${toAbsoluteTimestamp}**`;
};
var transformFresh = (builder) => {
  const { value } = builder.context;
  if (value.isUpcoming) {
    builder.setValue(
      "description",
      defaultTimestamp(value.liveTime.scheduledStart)
    );
    builder.setValue("footer", { text: "UPCOMING" });
  } else if (value.liveNow) {
    builder.setValue(
      "description",
      defaultTimestamp(value.liveTime.actualStart)
    );
    builder.setValue("footer", { text: "LIVE NOW" });
  } else {
    builder.setValue("description", defaultTimestamp(value.published));
    builder.setValue("footer", { text: "NEW UPLOAD" });
  }
};
var transformNotFresh = (builder) => {
  const { value, oldValue } = builder.context;
  if (oldValue !== void 0) {
    const getValues = (key) => ({
      current: value[key],
      previous: oldValue[key]
    });
    for (const k in value) {
      const key = k;
      switch (key) {
        case "liveNow": {
          const { current, previous } = getValues(key);
          if (current && !previous) {
            builder.setValue(
              "description",
              defaultTimestamp(value.liveTime.actualStart)
            );
            builder.setValue("footer", { text: "LIVE NOW" });
          } else if (!current && previous) {
            builder.setValue(
              "description",
              rangeTimestamp(
                value.liveTime.actualStart,
                value.liveTime.actualEnd
              )
            );
            builder.setValue("footer", { text: "WAS LIVE" });
          }
          break;
        }
        case "liveTime": {
          const { current, previous } = getValues(key);
          if (previous?.actualStart === void 0 && current?.actualEnd !== void 0) {
            builder.setValue(
              "description",
              rangeTimestamp(current.actualStart, current.actualEnd)
            );
            builder.setValue("footer", { text: "WAS LIVE" });
          }
          break;
        }
        case "title": {
          const { current, previous } = getValues(key);
          if (previous !== current) {
            builder.pushFields({
              name: key.toUpperCase(),
              value: `~~${previous}~~
${current}`
            });
            builder.setValueIf(
              "footer",
              { text: "UPDATED" },
              builder.body.footer === void 0
            );
          }
          break;
        }
        default:
          break;
      }
    }
  }
};

// src/discord/embed-builder/builder.ts
var DiscordEmbedBuilder = class {
  modified;
  body;
  context;
  constructor(context) {
    const { author, avatar, channelId, published, thumbnail, title, videoId } = context.value;
    this.body = {
      author: {
        name: author,
        icon_url: avatar,
        url: youtubeChannelLinkFromChannelId(channelId)
      },
      color: randomInt(0, 16777215),
      image: { url: thumbnail },
      timestamp: new Date(published),
      title,
      url: youtubeVideoLinkFromVideoId(videoId)
    };
    this.context = context;
    this.modified = false;
  }
  transform(transformer) {
    transformer(this);
  }
  getValue(key) {
    return this.body[key];
  }
  setValue(key, value) {
    this.body[key] = value;
    this.modified = true;
  }
  setValueIf(key, value, condition) {
    if (condition)
      this.setValue(key, value);
  }
  pushFields(...values) {
    this.modified = true;
    if (this.body.fields !== void 0)
      this.body.fields.push(...values);
    else
      this.body.fields = values;
  }
  unshiftFields(...values) {
    this.modified = true;
    if (this.body.fields !== void 0)
      this.body.fields.unshift(...values);
    else
      this.body.fields = values;
  }
};
var buildDiscordEmbed = ({ value, oldValue }) => {
  const builder = new DiscordEmbedBuilder({ value, oldValue });
  builder.transform(oldValue === void 0 ? transformFresh : transformNotFresh);
  if (!builder.modified)
    return;
  return builder.body;
};

// src/discord/index.ts
var buildDiscordWebhookMessages = (data, options) => {
  const embeds = data.map((it) => [buildDiscordEmbed(it), it]).filter(([embed]) => embed !== void 0).map(
    ([embed, it]) => options.hooks?.mapDiscordEmbed?.(embed, it) ?? embed
  );
  return chunked(embeds, 10).map((embedsChunk) => ({
    avatar_url: options.discord?.webhookAvatar,
    username: options.discord?.webhookUsername,
    embeds: embedsChunk
  })).map(
    (message) => options.hooks?.mapDiscordWebhookMessage?.(message) ?? message
  );
};
var sendDiscordWebhooks = async (messages, { webhookUrl }) => {
  const urls = Array.isArray(webhookUrl) ? webhookUrl : [webhookUrl];
  for (const message of messages) {
    for (const url of urls) {
      const response = await fetch(url, {
        body: JSON.stringify(message),
        headers: { "Content-Type": "application/json; charset=utf-8" },
        method: "POST"
      });
      if (!response.ok) {
        const text = await response.text();
        throw `Error (${response.status}) while sending webhook to ${url}: ${text}`;
      }
    }
  }
};

// src/index.ts
var handle = async (options) => {
  const url = new URL("https://youtube-feed-fn.netlify.app");
  for (const channel of options.channelIds) {
    url.searchParams.append("channels", channel);
  }
  const responses = await fetch(url.href).then((raw) => {
    return raw.json();
  });
  const filteredResponses = responses.filter(
    (value) => options.hooks?.filterResponse?.(value) ?? true
  );
  const { store, storeKey } = options;
  const responsesArchive = await store.get(storeKey).then((value) => JSON.parse(value ?? "{}"));
  const data = filteredResponses.map((value) => ({
    value,
    oldValue: responsesArchive[value.videoId]
  }));
  const webhookMessages = buildDiscordWebhookMessages(data, options);
  if (webhookMessages.length > 0) {
    await sendDiscordWebhooks(webhookMessages, options);
    for (const { value } of data) {
      responsesArchive[value.videoId] = value;
    }
    const archive = Object.fromEntries(
      Object.entries(responsesArchive).filter(
        ([_, value]) => options.hooks?.filterResponseArchive?.(value) ?? true
      )
    );
    await store.put(storeKey, JSON.stringify(archive));
  }
};
export {
  chunked,
  discordMention,
  discordUnixTimestamp,
  handle,
  randomInt,
  youtubeChannelLinkFromChannelId,
  youtubeVideoLinkFromVideoId
};
